/*
 *  jDTAUS Core RI Memory Manager
 *  Copyright (c) 2005 Christian Schulte
 *
 *  Christian Schulte, Haldener Strasse 72, 58095 Hagen, Germany
 *  <cs@jdtaus.org> (+49 2331 3543887)
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
package org.jdtaus.core.lang.ri;

import java.text.MessageFormat;
import java.util.Locale;
import org.jdtaus.core.container.ContainerError;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.ContainerInitializer;
import org.jdtaus.core.container.ContextFactory;
import org.jdtaus.core.container.ContextInitializer;
import org.jdtaus.core.container.Dependency;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.container.Properties;
import org.jdtaus.core.container.Property;
import org.jdtaus.core.container.PropertyException;
import org.jdtaus.core.lang.spi.MemoryManager;
import org.jdtaus.core.logging.spi.Logger;

/**
 * jDTAUS Core SPI {@code MemoryManager} reference implementation.
 * <p>The reference implementation leaves a configurable amount of memory free
 * and throws an {@code OutOfMemoryError} although the system has memory
 * available. This free memory is then available for proper exception handling
 * or for releasing resources in the system properly. It is configured with the
 * two configuration properties {@code maximumPercent} and
 * {@code maximumRetries}. Whenever an allocation would consume more than
 * {@code maximumPercent} percent of all available memory this implementation
 * tries to free memory by forcing garbage collection {@code maximumRetries}
 * times before throwing an {@code OutOfMemoryError} exception. The default for
 * property {@code maximumPercent} is {@code 98} and the default for property
 * {@code maximumRetries} is {@code 1}.</p>
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: DefaultMemoryManager.java 5002 2008-07-04 10:25:40Z schulte2005 $
 *
 * @see org.jdtaus.core.container.Container
 * @see #initialize()
 */
public final class DefaultMemoryManager
    implements MemoryManager, ContainerInitializer
{
    //--Implementation----------------------------------------------------------

// <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:jdtausImplementation
    // This section is managed by jdtaus-container-mojo.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(DefaultMemoryManager.class.getName());
// </editor-fold>//GEN-END:jdtausImplementation

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

// <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:jdtausConstructors
    // This section is managed by jdtaus-container-mojo.

    /**
     * <code>DefaultMemoryManager</code> implementation constructor.
     *
     * @param meta Implementation meta-data.
     *
     * @throws NullPointerException if <code>meta</code> is <code>null</code>.
     */
    private DefaultMemoryManager(final Implementation meta)
    {
        super();
        if(meta == null)
        {
            throw new NullPointerException("meta");
        }
        this.initializeProperties(meta.getProperties());
    }
    /**
     * <code>DefaultMemoryManager</code> dependency constructor.
     *
     * @param meta dependency meta-data.
     *
     * @throws NullPointerException if <code>meta</code> is <code>null</code>.
     */
    private DefaultMemoryManager(final Dependency meta)
    {
        super();
        if(meta == null)
        {
            throw new NullPointerException("meta");
        }
        this.initializeProperties(meta.getProperties());
    }

    /**
     * Initializes the properties of the instance.
     *
     * @param meta the property values to initialize the instance with.
     *
     * @throws NullPointerException if {@code meta} is {@code null}.
     */
    private void initializeProperties(final Properties meta)
    {
        Property p;

        if(meta == null)
        {
            throw new NullPointerException("meta");
        }

        p = meta.getProperty("maximumRetries");
        this.pMaximumRetries = ((java.lang.Integer) p.getValue()).intValue();


        p = meta.getProperty("maximumPercent");
        this.pMaximumPercent = ((java.lang.Long) p.getValue()).longValue();

    }
// </editor-fold>//GEN-END:jdtausConstructors

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------

    /**
     * Initializes the instance.
     *
     * @throws PropertyException for invalid property values.
     *
     * @see #assertValidProperties()
     */
    public void initialize()
    {
        this.assertValidProperties();
    }

    //----------------------------------------------------ContainerInitializer--
    //--Dependencies------------------------------------------------------------

// <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:jdtausDependencies
    // This section is managed by jdtaus-container-mojo.

    /** Configured <code>Logger</code> implementation. */
    private transient Logger dLogger;

    /**
     * Gets the configured <code>Logger</code> implementation.
     *
     * @return the configured <code>Logger</code> implementation.
     */
    private Logger getLogger()
    {
        Logger ret = null;
        if(this.dLogger != null)
        {
            ret = this.dLogger;
        }
        else
        {
            ret = (Logger) ContainerFactory.getContainer().
                getDependency(DefaultMemoryManager.class,
                "Logger");

            if(ModelFactory.getModel().getModules().
                getImplementation(DefaultMemoryManager.class.getName()).
                getDependencies().getDependency("Logger").
                isBound())
            {
                this.dLogger = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
// </editor-fold>//GEN-END:jdtausDependencies

    //------------------------------------------------------------Dependencies--
    //--Properties--------------------------------------------------------------

// <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:jdtausProperties
    // This section is managed by jdtaus-container-mojo.

    /**
     * Property {@code maximumRetries}.
     * @serial
     */
    private int pMaximumRetries;

    /**
     * Gets the value of property <code>maximumRetries</code>.
     *
     * @return the value of property <code>maximumRetries</code>.
     */
    public int getMaximumRetries()
    {
        return this.pMaximumRetries;
    }

    /**
     * Property {@code maximumPercent}.
     * @serial
     */
    private long pMaximumPercent;

    /**
     * Gets the value of property <code>maximumPercent</code>.
     *
     * @return the value of property <code>maximumPercent</code>.
     */
    public long getMaximumPercent()
    {
        return this.pMaximumPercent;
    }

// </editor-fold>//GEN-END:jdtausProperties

    //--------------------------------------------------------------Properties--
    //--MemoryManager-----------------------------------------------------------

    public long getAllocatedPercent()
    {
        final Runtime rt = Runtime.getRuntime();
        final long max = rt.maxMemory();
        final long limit = ( max / 100L ) * this.getMaximumPercent();
        return rt.totalMemory() * ( 100L / limit );
    }

    public long getAvailableBytes()
    {
        final Runtime rt = Runtime.getRuntime();
        final long all = rt.maxMemory() - rt.totalMemory() + rt.freeMemory();
        return ( all / 100L ) * this.getMaximumPercent();
    }

    public byte[] allocateBytes( int requested ) throws OutOfMemoryError
    {
        if ( requested < 0 )
        {
            throw new IllegalArgumentException( Integer.toString( requested ) );
        }

        byte[] ret = null;
        int retries = this.getMaximumRetries();

        do
        {
            final long available = this.getAvailableBytes();

            if ( available < requested )
            {
                this.logOutOfMemoryWarning( new Long( requested ),
                                            new Long( available ) );

                this.forceGarbageCollection(
                    this.getMaximumRetries() - retries );

            }
            else
            {
                ret = new byte[ requested ];
            }

        }
        while ( ret == null && retries-- > 0 );

        if ( ret == null )
        {
            throw new OutOfMemoryError();
        }

        return ret;
    }

    public long getAvailableShorts()
    {
        return this.getAvailableBytes() / 2;
    }

    public short[] allocateShorts( int requested ) throws OutOfMemoryError
    {
        if ( requested < 0 )
        {
            throw new IllegalArgumentException( Integer.toString( requested ) );
        }

        short[] ret = null;
        int retries = this.getMaximumRetries();

        do
        {
            final long available = this.getAvailableShorts();

            if ( available < requested )
            {
                this.logOutOfMemoryWarning( new Long( requested * 2 ),
                                            new Long( available * 2 ) );

                this.forceGarbageCollection(
                    this.getMaximumRetries() - retries );

            }
            else
            {
                ret = new short[ requested ];
            }

        }
        while ( ret == null && retries-- > 0 );

        if ( ret == null )
        {
            throw new OutOfMemoryError();
        }

        return ret;
    }

    public long getAvailableIntegers()
    {
        return this.getAvailableBytes() / 4;
    }

    public int[] allocateIntegers( int requested ) throws OutOfMemoryError
    {
        if ( requested < 0 )
        {
            throw new IllegalArgumentException( Integer.toString( requested ) );
        }

        int[] ret = null;
        int retries = this.getMaximumRetries();

        do
        {
            final long available = this.getAvailableIntegers();
            if ( available < requested )
            {
                this.logOutOfMemoryWarning( new Long( requested * 4 ),
                                            new Long( available * 4 ) );

                this.forceGarbageCollection(
                    this.getMaximumRetries() - retries );

            }
            else
            {
                ret = new int[ requested ];
            }


        }
        while ( ret == null && retries-- > 0 );

        if ( ret == null )
        {
            throw new OutOfMemoryError();
        }

        return ret;
    }

    public long getAvailableLongs()
    {
        return this.getAvailableBytes() / 8;
    }

    public long[] allocateLongs( int requested ) throws OutOfMemoryError
    {
        if ( requested < 0 )
        {
            throw new IllegalArgumentException( Integer.toString( requested ) );
        }

        long[] ret = null;
        int retries = this.getMaximumRetries();

        do
        {
            final long available = this.getAvailableLongs();

            if ( available < requested )
            {
                this.logOutOfMemoryWarning( new Long( requested * 8 ),
                                            new Long( available * 8 ) );

                this.forceGarbageCollection(
                    this.getMaximumRetries() - retries );

            }
            else
            {
                ret = new long[ requested ];
            }

        }
        while ( ret == null && retries-- > 0 );

        if ( ret == null )
        {
            throw new OutOfMemoryError();
        }

        return ret;
    }

    public long getAvailableChars()
    {
        return this.getAvailableBytes() / 2;
    }

    public char[] allocateChars( int requested ) throws OutOfMemoryError
    {
        if ( requested < 0 )
        {
            throw new IllegalArgumentException( Integer.toString( requested ) );
        }

        char[] ret = null;
        int retries = this.getMaximumRetries();

        do
        {
            final long available = this.getAvailableChars();

            if ( available < requested )
            {
                this.logOutOfMemoryWarning( new Long( requested * 2 ),
                                            new Long( available * 2 ) );

                this.forceGarbageCollection(
                    this.getMaximumRetries() - retries );

            }
            else
            {
                ret = new char[ requested ];
            }

        }
        while ( ret == null && retries-- > 0 );

        if ( ret == null )
        {
            throw new OutOfMemoryError();
        }

        return ret;
    }

    public long getAvailableFloats()
    {
        return this.getAvailableBytes() / 4;
    }

    public float[] allocateFloats( int requested ) throws OutOfMemoryError
    {
        if ( requested < 0 )
        {
            throw new IllegalArgumentException( Integer.toString( requested ) );
        }

        float[] ret = null;
        int retries = this.getMaximumRetries();

        do
        {
            final long available = this.getAvailableFloats();

            if ( available < requested )
            {
                this.logOutOfMemoryWarning( new Long( requested * 4 ),
                                            new Long( available * 4 ) );

                this.forceGarbageCollection(
                    this.getMaximumRetries() - retries );

            }
            else
            {
                ret = new float[ requested ];
            }

        }
        while ( ret == null && retries-- > 0 );

        if ( ret == null )
        {
            throw new OutOfMemoryError();
        }

        return ret;
    }

    public long getAvailableDoubles()
    {
        return this.getAvailableBytes() / 8;
    }

    public double[] allocateDoubles( int requested ) throws OutOfMemoryError
    {
        if ( requested < 0 )
        {
            throw new IllegalArgumentException( Integer.toString( requested ) );
        }

        double[] ret = null;
        int retries = this.getMaximumRetries();

        do
        {
            final long available = this.getAvailableDoubles();

            if ( available < requested )
            {
                this.logOutOfMemoryWarning( new Long( requested * 8 ),
                                            new Long( available * 8 ) );

                this.forceGarbageCollection(
                    this.getMaximumRetries() - retries );

            }
            else
            {
                ret = new double[ requested ];
            }

        }
        while ( ret == null && retries-- > 0 );

        if ( ret == null )
        {
            throw new OutOfMemoryError();
        }

        return ret;
    }

    public long getAvailableBooleans()
    {
        return this.getAvailableBytes();
    }

    public boolean[] allocateBoolean( int requested ) throws OutOfMemoryError
    {
        if ( requested < 0 )
        {
            throw new IllegalArgumentException( Integer.toString( requested ) );
        }

        boolean[] ret = null;
        int retries = this.getMaximumRetries();

        do
        {
            final long available = this.getAvailableBooleans();

            if ( available < requested )
            {
                this.logOutOfMemoryWarning( new Long( requested ),
                                            new Long( available ) );

                this.forceGarbageCollection(
                    this.getMaximumRetries() - retries );

            }
            else
            {
                ret = new boolean[ requested ];
            }

        }
        while ( ret == null && retries-- > 0 );

        if ( ret == null )
        {
            throw new OutOfMemoryError();
        }

        return ret;
    }

    //-----------------------------------------------------------MemoryManager--
    //--DefaultMemoryManager----------------------------------------------------

    /** Creates a new {@code DefaultMemoryManager} instance. */
    public DefaultMemoryManager()
    {
        this( META );
        this.initialize();
    }

    /**
     * Creates a new {@code DefaultMemoryManager} instance taking configuration
     * properties {@code maximumPercent} and {@code maximumRetries}.
     *
     * @throws PropertyException if property {@code maximumPercent} does not
     * match the range {@code [0, 100]} or property maximumRetries is negative.
     */
    public DefaultMemoryManager( final long maximumPercent,
                                  final int maximumRetries )
    {
        this( META );
        this.pMaximumPercent = maximumPercent;
        this.pMaximumRetries = maximumRetries;
        this.initialize();
    }

    /**
     * Checks configured properties.
     *
     * @throws PropertyException for illegal property values.
     */
    private void assertValidProperties() throws ContainerError
    {
        if ( this.getMaximumPercent() < 0L ||
            this.getMaximumPercent() > 100L )
        {
            throw new PropertyException(
                "maximumPercent", new Long( this.getMaximumPercent() ) );

        }
        if ( this.getMaximumRetries() < 0 )
        {
            throw new PropertyException(
                "maximumRetries", new Integer( this.getMaximumRetries() ) );

        }
    }

    /**
     * Logs a warning message for out of memory conditions.
     *
     * @param requestedByte number of byte requested to be allocated.
     * @param availableByte number of byte available when {@code requestedByte}
     * were requested.
     */
    private void logOutOfMemoryWarning( final Long requestedByte,
                                         final Long availableByte )
    {
        final MessageFormat fmt = DefaultMemoryManagerBundle.getInstance().
            getOutOfMemoryWarningMessage( Locale.getDefault() );

        this.getLogger().warn(
            fmt.format( new Object[] { requestedByte, availableByte } ) );

    }

    /**
     * Forces garbage collection and logs a warning message.
     *
     * @param repetition number of times garbage collection was already forced.
     *
     * @see System#gc()
     */
    private void forceGarbageCollection( final int repetition )
    {
        final MessageFormat fmt = DefaultMemoryManagerBundle.getInstance().
            getForcingGarbageCollectionMessage( Locale.getDefault() );

        this.getLogger().warn(
            fmt.format( new Object[] { new Integer( repetition ) } ) );

        System.gc();
    }

    //----------------------------------------------------DefaultMemoryManager--
}
