/*
 *  jDTAUS Core RI Memory Manager
 *  Copyright (c) 2005 Christian Schulte (cs@jdtaus.org)
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
package org.jdtaus.core.lang.ri;

import java.util.Locale;
import java.util.ResourceBundle;
import java.text.MessageFormat;

class DefaultMemoryManagerBundle
{

  /** Singleton instance. */
  private static final DefaultMemoryManagerBundle instance =
    new DefaultMemoryManagerBundle();

  /** Creates a new {@code DefaultMemoryManagerBundle} instance. */
  DefaultMemoryManagerBundle()
  {
    this.getMessage ( "forcingGarbageCollection", Locale.getDefault() );
    this.getMessage ( "outOfMemoryWarning", Locale.getDefault() );
  }

  /**
   * Gets the {@code DefaultMemoryManagerBundle} singleton.
   *
   * @return the {@code DefaultMemoryManagerBundle} singleton.
   */
  static DefaultMemoryManagerBundle getInstance()
  {
    return instance;
  }

  /**
   * Gets the text of the message with key {@code "forcingGarbageCollection"}.
   *
   * @return <pre>
   * Forcing garbage collection - already forced {0, number} times.</pre>
   */
  String getForcingGarbageCollectionText ( final Locale locale )
  {
    return this.getMessage ( "forcingGarbageCollection", locale );
  }

  /**
   * Gets the message with key {@code "forcingGarbageCollection"}.
   *
   * @return <pre>
   * Forcing garbage collection - already forced {0, number} times.</pre>
   */
  MessageFormat getForcingGarbageCollectionMessage ( Locale locale )
  {
    if ( locale == null )
    {
      locale = Locale.getDefault();
    }

    return new MessageFormat ( this.getMessage ( "forcingGarbageCollection", locale ), locale );
  }
  /**
   * Gets the text of the message with key {@code "outOfMemoryWarning"}.
   *
   * @return <pre>
   * Running out of memory. Needed {0, number} byte when {1, number} were available.</pre>
   */
  String getOutOfMemoryWarningText ( final Locale locale )
  {
    return this.getMessage ( "outOfMemoryWarning", locale );
  }

  /**
   * Gets the message with key {@code "outOfMemoryWarning"}.
   *
   * @return <pre>
   * Running out of memory. Needed {0, number} byte when {1, number} were available.</pre>
   */
  MessageFormat getOutOfMemoryWarningMessage ( Locale locale )
  {
    if ( locale == null )
    {
      locale = Locale.getDefault();
    }

    return new MessageFormat ( this.getMessage ( "outOfMemoryWarning", locale ), locale );
  }

  /**
   * Gets a message for a given key and locale.
   *
   * @param key the key of the message to return.
   * @param locale the locale of the message.
   *
   * @return message for {@code key} using {@code locale}.
   */
  private String getMessage( final String key, Locale locale )
  {
    if ( locale == null )
    {
      locale = Locale.getDefault();
    }

    return ResourceBundle.getBundle( "org/jdtaus/core/lang/ri/DefaultMemoryManager",
                                     locale ).getString( key );

  }
}
